SuperStrict

Framework BRL.Max2D
Import BRL.D3D7Max2D
Import BRL.Retro
Import BRL.PNGLoader

SeedRnd MilliSecs()
AppTitle$ = "32 Hebauf"
SetGraphicsDriver D3D7Max2DDriver()
Graphics 640, 400, 0, 60
SetBlend ALPHABLEND
SetClsColor 120, 180, 80
SetMaskColor 100, 200, 0
AutoMidHandle True

Const PHASE_INTRO% = 0, PHASE_GIVE% = 1, PHASE_TAKE% = 2
Global Phase%, DeckList:TList = New TList, CardList:TList = New TList, TakenList:TList = New TList, IntroProg%, Time%, SelCard:TCard, Bow#
Global Background:TImage, ImageFont:TImage, PictoTop:TImage, PictoGround:TImage, LogoFit:TImage, LogoKarten:TImage

Background = LoadImage("graphics/background.png")
ImageFont = LoadAnimImage("graphics/numbers.png", 32, 64, 0, 10)
PictoTop = LoadImage("graphics/pictotop.png")
PictoGround = LoadImage("graphics/pictoground.png")
LogoFit = LoadImage("graphics/fit.png")
LogoKarten = LoadImage("graphics/karten.png")
TCard.Load()
Init()

Repeat
Local card:TCard
For card = EachIn CardList
card.Update()
Next

Select Phase
Case PHASE_INTRO
IntroProg:+ 1
If IntroProg > 180 Then Phase = PHASE_GIVE
Case PHASE_GIVE
If MouseHit(MOUSE_LEFT) And MouseDown(MOUSE_LEFT) Then
If DeckList.IsEmpty() Then
Phase = PHASE_TAKE
Time = MilliSecs()
Else
card = TCard(DeckList.First())
DeckList.RemoveFirst()
If Rnd(1000) > 500 Then card.Covered = True
card.PosX = MouseX()
card.PosY = MouseY()
card.DestX = 40 + Rnd(560)
card.DestY = 40 + Rnd(320)

CardList.AddLast(card)
EndIf
EndIf
Case PHASE_TAKE
If CardList.IsEmpty() Then
If Proceed("Glckwunsch, Sie haben alle 32 Karten in nur " + Int((MilliSecs() - Time) * 0.001) + " Sekunden aufgehoben.~nMchten Sie es vielleicht noch einmal versuchen?~n") = 1 Then
Init()
Else
End
EndIf
EndIf

If MouseHit(MOUSE_LEFT) Then
ResetCollisions()
For card = EachIn CardList
card.Collide()
Next

Local cards:TCard[] = TCard[](CollideRect(MouseX(), MouseY(), 1, 1, 1, 0))
If cards <> Null Then SelCard = cards[0] Bow = 0
EndIf

If SelCard <> Null Then
Bow:* 0.99
If KeyHit(KEY_SPACE) And KeyDown(KEY_SPACE) Then Bow:+ 0.2
If Bow > 1 Then
SelCard.DestY = - 200
TakenList.AddLast(SelCard)
CardList.Remove(SelCard)
SelCard = Null
EndIf
EndIf
End Select
Cls
TileImage Background, 0, 0

For card = EachIn CardList
card.Render()
Next
For card = EachIn TakenList
card.Update()
card.Render()
If card.PosY < - 100 Then TakenList.Remove(card)
Next

If SelCard <> Null Then RenderPicto(Bow, SelCard.PosX + 40, SelCard.PosY - 40)

If Phase = PHASE_INTRO Then
SetColor 0, 0, 0
SetAlpha 0.3
If IntroProg > 160 Then SetAlpha 0.3 * ((180 - IntroProg) / 20.0)
DrawRect 0, 160, 640, 80
SetAlpha 1
SetColor 255, 255, 255
If IntroProg < 60 Then
DrawImage LogoFit, IntroProg * 4.5 - 64, 180
DrawImage LogoKarten, 640 - IntroProg * 4, 220
ElseIf IntroProg > 120 Then
DrawImage LogoFit, IntroProg * 12 - 1234, 180
DrawImage LogoKarten, 640 - IntroProg * 12 + 1200, 220
Else
DrawImage LogoFit, 206, 180
DrawImage LogoKarten, 400, 220
EndIf
Else
If DeckList.Count() > 0 Then DrawImage TCard.Sprite, MouseX(), MouseY(), 32
RenderNumber CardList.Count(), 320, 32
EndIf
Flip
Until AppTerminate()
End

Function Init()
DeckList.Clear()
For Local i% = 0 Until 32
Local card:TCard = TCard.Create(i)
card.ShuffleID = Rnd(1000)
DeckList.AddLast(TCard.Create(i))
Next
DeckList.Sort(True)
CardList.Clear()
Phase = PHASE_INTRO
End Function

Function RenderNumber(num%, x#, y#)
Local number$ = String(num)

For Local i% = 0 Until Len(number)
DrawImage ImageFont, x + i * 24, y, Int(Chr(number[i]))
Next
End Function

Function RenderPicto(b#, x#, y#)
SetColor 0, 0, 0
SetRotation - 90 * b
DrawImage PictoTop, x, y
SetRotation 0
DrawImage PictoGround, x, y
SetColor 255, 255, 255
End Function

Type TCard
Global Sprite:TImage
Field ID%, Covered:Byte, Rot#, PosX#, PosY#, DestX#, DestY#, ShuffleID%

Function Create:TCard(id%)
Local created:TCard = New TCard
created.ID = id
Return created
End Function

Function Load()
Sprite = LoadAnimImage("graphics/cards.png", 73, 98, 0, 33)
End Function

Method Compare%(other:Object)
If other = Null Return - 1
If TCard(other) = Self Then Return 0
If Self.ShuffleID > TCard(other).ShuffleID Then
Return 1
Else
Return - 1
EndIf
End Method

Method Collide()
SetRotation Rot
CollideImage Sprite, Self.PosX, Self.PosY, Self.ID, 0, 1, Self
SetRotation 0
End Method

Method Update()
Local movex# = Self.DestX - Self.PosX, movey# = Self.DestY - Self.PosY
Local length# = Sqr(movex * movex + movey * movey)

Self.Rot = (Rot + length * 0.04) Mod 360
If length < 8 Then
Self.PosX = Self.DestX
Self.PosY = Self.DestY
Else
length:/ 8
Self.PosX:+ movex / length
Self.PosY:+ movey / length
EndIf
End Method

Method Render()
Local frame% = Self.ID
If Self.Covered Then frame = 32
SetRotation Rot
DrawImage Sprite, Self.PosX, Self.PosY, frame
SetRotation 0
End Method
End Type
